// an example of using the libmcss api
// loads then flattens a model then outputthe flattened representation
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#include <stdio.h>
#include <mcss/mcss.h>

using namespace Simulation;

int main(int argc, char **argv)
{
	unsigned long i, j, num_species, num_reactions;
	std::vector< int > initial_amounts, reactants, products;

	// parse command line and read in parameters
	SimulationParameters parameters;
	parameters.read(argc, argv);

	// create p-system from parameters specified in parameter file
	Psystem psystem(parameters);

	// flatten model
	psystem.flatten();

	// output model information
	// number of species
	num_species = psystem.getNumSpeciesFlat();
	printf("num_species=%u\n", num_species);
	
	// number of reactions
	num_reactions = psystem.getNumReactionsFlat();
	printf("num_reactions=%u\n", num_reactions);

	// initial species level
	initial_amounts = psystem.getInitialSpeciesFlat();
	for(i = 0; i < num_species; i++)
		printf("species=%u initial_amount=%u\n", i, initial_amounts[i]);
	
	// reactions
	for(i = 0; i < num_reactions; i++) {
		// reaction constant
		printf("rule=%u reaction_constant=%0.8E\n", i, \
				psystem.getReactionConstantFlat(i));
		
		// reactants
		reactants = psystem.getReactantsFlat(i);
		printf("rule=%u reactants=[", i);
		for(j = 0; j < reactants.size(); j++)
			printf(" %u", reactants[j]);
		printf(" ]\n");

		// products
		products = psystem.getProductsFlat(i);
		printf("rule=%u products=[", i);
		for(j = 0; j < products.size(); j++)
			printf(" %u", products[j]);
		printf(" ]\n");
	}

	return 0;
}
