// alphabet class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef ALPHABET_H
#define ALPHABET_H

#include <string>
#include <list>
#include <sbml/SBMLTypes.h>

namespace Simulation {

// alphabet class
class Alphabet {
		unsigned long alphabet_size;  // number of objects
		std::string **name;  // names of objects
		bool *constant;  // flag indicating if species is constant
		bool *distribution;  // flag indicating if species distribution-based
		unsigned int *distribution_type;  // type of distribution
		double *distribution_mean;  // distribution mean
		double *distribution_sd;  // distribution standard deviation
		bool *dividing;  // flag indicating if species divides during division

	public:
		// constructor
		Alphabet(unsigned long);

		// destructor
		~Alphabet();

		// general access methods
		unsigned long size(void);
		std::string getName(unsigned long);
		void setName(unsigned long, std::string);
		unsigned long getSpeciesIndex(std::string);
		bool isDegradation(std::string);
		bool isDistribution(unsigned long);
		unsigned long getLevel(unsigned long);
		void setConstant(unsigned long);
		bool isConstant(unsigned long);
		void setNondividing(unsigned long);
		bool isNondividing(unsigned long);
		double getDistributionMean(unsigned long);

		// add species to alphabet
		void addSpecies(std::string name);
};
}

#endif
