// deterministic ODE solver class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef DETERMINISTICODE_H
#define DETERMINISTICODE_H

#include "Psystem.h"
#include <list>
#include <gsl/gsl_odeiv.h>

namespace Simulation {

// information on deterministic variable
typedef struct {
	unsigned long compartment;  // compartment index
	unsigned long ruleset;  // ruleset index
	unsigned long rule;  // rule index
	unsigned long species[2];  // species index
	unsigned long variable[2];  // variable index
	double factor;  // reactant (-1) or product (1) and dimeric factor
} ODEVariableEntry;

typedef struct {
	unsigned long compartment;  // compartment index
	unsigned long species;  // species index
} ODEVariableLookupEntry;

// determinisitic ODE solver
class DeterministicODE : public SimulationAlgorithm {
		unsigned int num_variables;  // number of ODE variables
		vector<ODEVariableEntry> **ode1; // first order odes
		vector<ODEVariableEntry> **ode2; // first order odes
		// lookup table of ODE variables to compartment species
		ODEVariableLookupEntry *variable_lookup;
		double *ode_state;  // initial ode state
		double time_offset;  // ode solver time offset for events
		double ode_step_size;  // ode solver step size

		// ode solver
		gsl_odeiv_step *ode_stepper;  // stepping function
		gsl_odeiv_control *ode_control;  // ode control object
		gsl_odeiv_evolve *ode_evolution;  // ode evolution function
		gsl_odeiv_system ode_system;  // ode system definition

	public:
		// constructor
		DeterministicODE(const unsigned long seed = 0);

		// destructor
		~DeterministicODE();

		// perform one iteration of algorithm for all compartments
		int execute(void);

		// perform one iteration of algorithm for one compartment
		void execute(unsigned long);

		// initialise algorithm
		int preprocess(void);

		// general access methods
		Psystem *getPsystem(void);
		unsigned long getNumVariables(void);
		vector<ODEVariableEntry> **getODE1(void);
		vector<ODEVariableEntry> **getODE2(void);
		ODEVariableLookupEntry *getVariableLookup(void);
};

}

#endif
