// event class definitions
// copyright 2008, 2009, 2010 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef EVENT_H
#define EVENT_H

#include <string>
#include <muParser.h>

#include "Alphabet.h"

namespace Simulation {

// event class
class Event {
		std::string id;  // id of event template
		std::string name;  // name of event template
		unsigned long index;  // index of event template
		bool fired;  // flag indicating if trigger fired
		mu::Parser trigger_function;  // trigger function
		double time;  // time copy
		double volume, initial_volume;  // compartment volume
		unsigned long num_trigger_species;  // num of trigger species copies
		unsigned long *trigger_species;  // copies of trigger species indices
		double *trigger_species_value;  // copies of trigger species levels
		std::string assignment_variable;  // event assignment variable
		unsigned long assignment_variable_index;  // variable index
		mu::Parser assignment_function;  // event assignment function
		unsigned long num_assignment_species;  // num assignment species copies
		unsigned long *assignment_species;  // copies assignment species indices
		double *assignment_species_value;  // copies assignment species levels

	public:
		// constructor
		Event(std::string);

		// destructor
		~Event();

		// copy constructor
		Event(const Event &);

		// general access methods
		void setId(std::string);
		std::string getId(void);
		void setName(std::string);
		std::string getName(void);
		void setIndex(unsigned long);
		unsigned long getIndex(void);
		void setTime(double);
		double getTime(void);
		void setVolume(double);
		double getVolume(void);
		void setInitialVolume(double);
		double getInitialVolume(void);
		void setNumTriggerSpecies(unsigned long);
		unsigned long getNumTriggerSpecies(void);
		void setTriggerSpecies(unsigned long, unsigned long);
		unsigned long getTriggerSpecies(unsigned long);
		void setTriggerSpeciesValue(unsigned long, unsigned long);
		unsigned long getTriggerSpeciesValue(unsigned long);
		void setNumAssignmentSpecies(unsigned long);
		unsigned long getNumAssignmentSpecies(void);
		void setAssignmentSpecies(unsigned long, unsigned long);
		unsigned long getAssignmentSpecies(unsigned long);
		void setAssignmentSpeciesValue(unsigned long, unsigned long);
		unsigned long getAssignmentSpeciesValue(unsigned long);
		unsigned long getAssignmentVariableIndex(void);

		// set trigger function
		void setTriggerFunction(std::string n, Alphabet *alphabet);

		// set assignment function
		void setAssignmentFunction(std::string v, std::string n, \
				Alphabet *alphabet);

		// evaluate trigger function
		bool evaluateTrigger(void);

		// evaluate assignment function
		unsigned long evaluateAssignment(void);
};

}

#endif
