// parameter class definitions
// automatically generated by make-parameter-class
// part of libecsb - the executable cell systems biology library
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef MCSSCMAESPARAMETERS_H
#define MCSSCMAESPARAMETERS_H

#include <ecsb/ecsb.h>

// McssCmaesParameters class
class McssCmaesParameters : public ECSB::ParameterSet {
	public:
		// constructor
		McssCmaesParameters(void);

		// parameters
		// number of individuals in population (lambda)
		int population_size;
		// number of parameter optimisation runs to perform
		int optimisation_runs;
		// number of parameters (N)
		int num_parameters;
		// maximum number of iterations (stopMaxIter)
		double max_iterations;
		// maximum number of fitness function evaluations (stopMaxFunEvals)
		double max_function_evals;
		// stop if fitness is less than given value (stopFitness)
		double stop_fitness;
		// stop if fitness differences are less than given value (stopTolFun)
		double stop_fitness_diff;
		// stop if fitness differences of best individuals are less than given value (stopTolFunHist)
		double stop_fitness_diff_best;
		// stop if parameter steps are less than given value (stopTolX)
		double stop_step_size;
		// stop if standard deviation is greater than given value (stopTolUpXFactor)
		double stop_sd;
		// random number seed (seed)
		unsigned int seed;
		// maximum CPU time fraction for eigensystem decomposition (maxTimeFractionForEigendecompostion)
		double max_eigen_decomposition_time;
		// number of generations before updating eignesystem (updatecov)
		double updatecov;
		// multiplier for updatecov (fac*updatecov)
		double updatecov_multiplier;
		// filename of restart distribution to read (resume)
		char resume_distribution[PARAMETER_STR_MAX];
		// filename to saved parameters actually used to (actparcmaes.par)
		char used_parameter_file[PARAMETER_STR_MAX];
		// (weights)
		char weights[PARAMETER_STR_MAX];
		// (mu)
		int mu;
		// (fac*cs)
		double fac_cs;
		// (fac*damps)
		double fac_damps;
		// (ccumcov)
		double ccumcov;
		// (mucov)
		double mucov;
		// (fac*ccov)
		double fac_ccov;
		// (diagonalCovarianceMatrix)
		double diagonal_covariance_matrix;
		// (fac*maxFunEvals)
		double fac_max_func_evals;
		// file containing user defined functions
		char function_file[PARAMETER_STR_MAX];
		// initial optimisation starting point
		char initial_x[PARAMETER_STR_MAX];
		// mcss parameter filename
		char mcss_parameter_file[PARAMETER_STR_MAX];
		// perform individual mcss runs in parallel
		bool parallel_runs;
		// lower bounds for parameter values
		double lower_bound;
		// upper bounds for parameter values
		double upper_bound;
		// separator for input data columns
		char column_separator;
		// file containing target data
		char target_data_file[PARAMETER_STR_MAX];
		// file tp save optimisation results to
		char data_file[PARAMETER_STR_MAX];
		// compress hdf5 output
		bool compress;
		// hdf5 compression level (0-9; 9=best)
		unsigned int compression_level;
		// output current log interval to screen
		bool show_progress;
		// number of simulation runs to perform for each individual (overrides mcss runs parameter)
		unsigned long runs;
		// number of simulation runs to perform for each individual if performing subruns (overrides mcss runs parameter)
		unsigned long subruns;
		// maximum wallclock time to run simulation (overrides mcss max_runtime parameter)
		double max_runtime;

		// set a parameter value
		bool setParameter(char *name, char *value);

		// write parameter set to file
		void write(const char *filename);
};

#endif
