// rule class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef RULE_H
#define RULE_H

#include <string>
#include "Alphabet.h"

namespace Simulation {

// rule class
class Rule {
		double c;  // stochastic reaction constant
		std::string id;  // id of rule
		std::string name;  // name of rule
		long int vector[3];  // rule vector
		std::string target_name;  // rule target name
		std::string target_type;  // rule target name
		std::string constant_name;  // name of constant
		double propensity;  // rule propensity
		bool channel;  // flag indicating if rule is translocation rule
		unsigned long num_reactants;  // number of reactants
		unsigned long reactant[2];  // index of reactant
		unsigned long num_products;  // number of products
		unsigned long product[3];  // index of product
		unsigned long initial_reactant[2];  // initial amounts of reactants
		unsigned long initial_product[3];  // initial amounts of reactants
		double D;  // diffusion constant
		double h;  // length of diffusion subvolume
		unsigned int N;  // number of diffusion subvolumes
		bool diffusion;  // flag indicating if diffusion rule
		double br;  // base rate of diffusion
		double st;  // diffusion start time
		bool distribution;  // flag indicating if constant distribution-based
		unsigned int distribution_type;  // type of distribution
		double distribution_mean;  // distribution mean
		double distribution_sd;  // distribution standard deviation

	public:
		// constructor
		Rule(std::string);

		// destructor
		~Rule();

		// general access methods
		void setReactionConstant(double);
		double getReactionConstant(void);
		void setReactionConstantName(std::string);
		std::string getReactionConstantName(void);
		void setXVector(long int);
		void setYVector(long int);
		void setZVector(long int);
		long int getXVector(void);
		long int getYVector(void);
		long int getZVector(void);
		void show(void);
		void setPropensity(double);
		double getPropensity(void);
		int isChannel(void);
		void setChannel(void);
		unsigned long getReactantIndex(unsigned long);
		void setReactantIndex(unsigned long, unsigned long);
		unsigned long getProductIndex(unsigned long);
		void setProductIndex(unsigned long, unsigned long);
		std::string getId(void);
		void setId(std::string);
		std::string getName(void);
		void setName(std::string);
		const char *getNameStr(void);
		unsigned long getNumReactants(void);
		void setNumReactants(unsigned long);
		unsigned long getNumProducts(void);
		void setNumProducts(unsigned long);
		void setDiffusion(void);
		bool isDiffusion(void);
		void setDiffusionConstant(double);
		double getDiffusionConstant(void);
		void setDiffusionLength(double);
		double getDiffusionLength(void);
		void setDiffusionNumber(unsigned int);
		unsigned int getDiffusionNumber(void);
		void setDiffusionBaseRate(double);
		double getDiffusionBaseRate(void);
		void setDiffusionStart(double);
		double getDiffusionStart(void);
		void setDistribution(std::string);
		bool isDistribution(void);
		void setTargetName(std::string);
		std::string getTargetName(void);
		void setTargetType(std::string);
		std::string getTargetType(void);
		void setInitialReactant(unsigned long, unsigned long);
		unsigned long getInitialReactant(unsigned long);
		void setInitialProduct(unsigned long, unsigned long);
		unsigned long getInitialProduct(unsigned long);
		std::string getRuleAsString(Alphabet *);
};

}

#endif
