// simulation datasets definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#include "SimulationDatasets.h"

#include <iostream>
#include <stdlib.h>

using namespace SimulationDatasets;

///////////////////////////////////
// "species_information" dataset //
///////////////////////////////////

// create species dataset datatype
hid_t SimulationDatasets::createSpeciesDatatype(void)
{
	hid_t dtype;

	// initialise datatype
	if((dtype = H5Tcreate(H5T_COMPOUND, (size_t) sizeof(SpeciesEntry))) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	// insert members
	if(H5Tinsert(dtype, "species_index", HOFFSET(SpeciesEntry, si), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	hid_t dspace = H5Screate(H5S_SCALAR);
	hid_t stype = H5Tcopy(H5T_C_S1);
	H5Tset_size(stype, 255);
	H5Tset_strpad(stype, H5T_STR_NULLTERM);
	if(H5Tinsert(dtype, "species_name", HOFFSET(SpeciesEntry, sn), \
			stype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	H5Tclose(stype);
	H5Sclose(dspace);

	return dtype;
}

///////////////////////////////////////
// "compartment_information" dataset //
///////////////////////////////////////

// create compartment dataset datatype
hid_t SimulationDatasets::createCompartmentDatatype(void)
{
	hid_t dtype;

	// initialise datatype
	if((dtype = H5Tcreate(H5T_COMPOUND, (size_t) sizeof(CompartmentEntry))) \
			< 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	// insert members
	if(H5Tinsert(dtype, "compartment_index", HOFFSET(CompartmentEntry, ci), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	hid_t dspace = H5Screate(H5S_SCALAR);
	hid_t stype = H5Tcopy(H5T_C_S1);
	H5Tset_size(stype, 255);
	H5Tset_strpad(stype, H5T_STR_NULLTERM);
	if(H5Tinsert(dtype, "compartment_id", HOFFSET(CompartmentEntry, cid), \
			stype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	H5Tset_size(stype, 255);
	if(H5Tinsert(dtype, "compartment_name", HOFFSET(CompartmentEntry, cn), \
			stype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "compartment_x_position", \
			HOFFSET(CompartmentEntry, cx), H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "compartment_y_position", \
			HOFFSET(CompartmentEntry, cy), H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "compartment_template_index", \
			HOFFSET(CompartmentEntry, ct), H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "compartment_creation_time", \
			HOFFSET(CompartmentEntry, cst), H5T_NATIVE_DOUBLE) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "compartment_destruction_time", \
			HOFFSET(CompartmentEntry, cet), H5T_NATIVE_DOUBLE) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	H5Tclose(stype);
	H5Sclose(dspace);

	return dtype;
}

////////////////////////////////
// "rule_information" dataset //
////////////////////////////////

// create rule dataset datatype
hid_t SimulationDatasets::createRuleDatatype(void)
{
	hid_t dtype;

	// initialise datatype
	if((dtype = H5Tcreate(H5T_COMPOUND, (size_t) sizeof(RuleEntry))) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	// insert members
	if(H5Tinsert(dtype, "rule_template_index", HOFFSET(RuleEntry, rsi), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "rule_index", HOFFSET(RuleEntry, ri), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	hid_t dspace = H5Screate(H5S_SCALAR);
	hid_t stype = H5Tcopy(H5T_C_S1);
	H5Tset_size(stype, 255);
	H5Tset_strpad(stype, H5T_STR_NULLTERM);
	if(H5Tinsert(dtype, "rule_id", HOFFSET(RuleEntry, rid), \
			stype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	H5Tset_size(stype, 255);
	if(H5Tinsert(dtype, "rule_name", HOFFSET(RuleEntry, rn), \
			stype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "rule_x_target", HOFFSET(RuleEntry, rx), \
			H5T_NATIVE_LONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "rule_y_target", HOFFSET(RuleEntry, ry), \
			H5T_NATIVE_LONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	H5Tset_size(stype, 255);
	if(H5Tinsert(dtype, "rule_string", HOFFSET(RuleEntry, rstr), \
			stype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	H5Tclose(stype);
	H5Sclose(dspace);

	return dtype;
}

///////////////////////////////////
// "ruleset_information" dataset //
///////////////////////////////////

// create ruleset dataset datatype
hid_t SimulationDatasets::createRulesetDatatype(void)
{
	hid_t dtype;

	// initialise datatype
	if((dtype = H5Tcreate(H5T_COMPOUND, (size_t) sizeof(RulesetEntry))) \
			< 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	// insert members
	if(H5Tinsert(dtype, "ruleset_index", HOFFSET(RulesetEntry, rsi), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	hid_t dspace = H5Screate(H5S_SCALAR);
	hid_t stype = H5Tcopy(H5T_C_S1);
	H5Tset_size(stype, 255);
	H5Tset_strpad(stype, H5T_STR_NULLTERM);
	if(H5Tinsert(dtype, "ruleset_name", HOFFSET(RulesetEntry, rsn), \
			stype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	H5Tset_size(stype, 255);
	if(H5Tinsert(dtype, "ruleset_compartment_id", HOFFSET(RulesetEntry, rsc), \
			stype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "number_of_rules_in_ruleset", \
			HOFFSET(RulesetEntry, nr), H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	H5Tclose(stype);
	H5Sclose(dspace);

	return dtype;
}

/////////////////////////
// "reactions" dataset //
/////////////////////////

// create reactions dataset datatype
hid_t SimulationDatasets::createReactionDatatype(void)
{
	hid_t dtype;

	// initialise datatype
	if((dtype = H5Tcreate(H5T_COMPOUND, (size_t) sizeof(ReactionEntry))) \
			< 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	// insert members
	if(H5Tinsert(dtype, "time", HOFFSET(ReactionEntry, t), \
			H5T_NATIVE_DOUBLE) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "species_index", HOFFSET(ReactionEntry, si), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "species_amount", HOFFSET(ReactionEntry, sl), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "compartment_index", HOFFSET(ReactionEntry, ci), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "rule_template_index", HOFFSET(ReactionEntry, rsi), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "rule_index", HOFFSET(ReactionEntry, ri), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	return dtype;
}

////////////////////////////
// "propensities" dataset //
////////////////////////////

// create propensities dataset datatype
hid_t SimulationDatasets::createPropensityDatatype( \
		const unsigned long total_rules)
{
	hid_t dtype;

	// initialise datatype
	size_t size = doubleOffset(1) + 3 * uLongOffset(total_rules) + \
				doubleOffset(total_rules);
	if((dtype = H5Tcreate(H5T_COMPOUND, size)) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	size = 0;
	if(H5Tinsert(dtype, "time", size, H5T_NATIVE_DOUBLE) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	size += doubleOffset(1);
	hsize_t dims[1] = { total_rules };
	hid_t atype = H5Tarray_create2(H5T_NATIVE_ULONG, 1, dims);
	if(H5Tinsert(dtype, "rule_index", size, atype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	size += uLongOffset(total_rules);
	if(H5Tinsert(dtype, "ruleset_index", size, atype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	size += uLongOffset(total_rules);
	if(H5Tinsert(dtype, "compartment_index", size, atype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	size += uLongOffset(total_rules);
	H5Tclose(atype);
	atype = H5Tarray_create2(H5T_NATIVE_DOUBLE, 1, dims);
	if(H5Tinsert(dtype, "rule_propensity", size, atype) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	H5Tclose(atype);

	return dtype;
}

// convert propensities data entry to structure
void SimulationDatasets::convertPropensityEntry(void *buffer, \
		hid_t dtype, PropensityEntry &entry, const unsigned long index)
{
	// get entry in buffer
	DS_BYTE *row = ((DS_BYTE *) buffer) + index * H5Tget_size(dtype);

	// set structure variables
	entry.t = (double *) (row + \
			getEntryOffset(dtype, "time"));
	entry.ri = (unsigned long *) (row + \
			getEntryOffset(dtype, "rule_index"));
	entry.rsi = (unsigned long *) (row + \
			getEntryOffset(dtype, "ruleset_index"));
	entry.ci = (unsigned long *) (row + \
			getEntryOffset(dtype, "compartment_index"));
	entry.rp = (double *) (row + \
			getEntryOffset(dtype, "rule_propensity"));
}

///////////////////////
// "amounts" dataset //
///////////////////////

// create amounts dataset datatype
hid_t SimulationDatasets::createAmountDatatype(void)
{
	hid_t dtype;

	if((dtype = H5Tcopy(H5T_NATIVE_UINT)) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	return dtype;
}

///////////////////////
// "volumes" dataset //
///////////////////////

// create volumes dataset datatype
hid_t SimulationDatasets::createVolumeDatatype(void)
{
	hid_t dtype;

	if((dtype = H5Tcopy(H5T_NATIVE_DOUBLE)) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	return dtype;
}

/////////////////////////
// "positions" dataset //
/////////////////////////

// create positions dataset datatype
hid_t SimulationDatasets::createPositionDatatype(void)
{
	hid_t dtype;

	// initialise datatype
	if((dtype = H5Tcreate(H5T_COMPOUND, (size_t) sizeof(PositionEntry))) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	// insert members
	if(H5Tinsert(dtype, "compartment_index", HOFFSET(PositionEntry, ci), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "compartment_x_position", HOFFSET(PositionEntry, cx), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "compartment_y_position", HOFFSET(PositionEntry, cy), \
			H5T_NATIVE_ULONG) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}
	if(H5Tinsert(dtype, "compartment_creation_time", \
			HOFFSET(PositionEntry, ct), H5T_NATIVE_DOUBLE) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	return dtype;
}

////////////////////////////
// "lattice_spin" dataset //
////////////////////////////

// create lattice spin dataset datatype
hid_t SimulationDatasets::createLatticeSpinDatatype(void)
{
	hid_t dtype;

	if((dtype = H5Tcopy(H5T_NATIVE_UINT)) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	return dtype;
}

////////////////////////////
// "lattice_type" dataset //
////////////////////////////

// create lattice type dataset datatype
hid_t SimulationDatasets::createLatticeTypeDatatype(void)
{
	hid_t dtype;

	if((dtype = H5Tcopy(H5T_NATIVE_UINT)) < 0) {
		std::cerr << "error: couldn't create datatype\n";
		exit(1);
	}

	return dtype;
}
