// parameter class definitions
// automatically generated by make-parameter-class
// part of libecsb - the executable cell systems biology library
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef SIMULATIONPARAMETERS_H
#define SIMULATIONPARAMETERS_H

#include <ecsb/ecsb.h>

// SimulationParameters class
class SimulationParameters : public ECSB::ParameterSet {
	public:
		// constructor
		SimulationParameters(void);

		// parameters
		// filename of sbml/xml model
		char model_file[PARAMETER_STR_MAX];
		// filename of xml lattice
		char lattice_file[PARAMETER_STR_MAX];
		// maximum time to run simulation
		double max_time;
		// maximum wallclock time to run simulation
		double max_runtime;
		// file to save simulation data to
		char data_file[PARAMETER_STR_MAX];
		// simulation algorithm to use
		char simulation_algorithm[PARAMETER_STR_MAX];
		// type of data logging to perform
		char log_type[PARAMETER_STR_MAX];
		// interval to log data
		double log_interval;
		// log output to memory
		bool log_memory;
		// duplicate initial amounts for all templates
		bool duplicate_initial_amounts;
		// random number seed
		unsigned long seed;
		// log reaction propensities
		bool log_propensities;
		// log compartment volumes
		bool log_volumes;
		// rate of volume change
		unsigned long volume_rate;
		// rate of division change
		unsigned long division_rate;
		// rate of cellular potts update
		unsigned long potts_rate;
		// log up to max_time if steady state reached
		bool log_steady_state;
		// log lattice
		bool log_lattice;
		// lattice x periodic boundary condition
		bool periodic_x;
		// lattice y periodic boundary condition
		bool periodic_y;
		// lattice z periodic boundary condition
		bool periodic_z;
		// log levels of degraded species
		bool log_degraded;
		// just initialise p-system - no simulation
		bool just_psystem;
		// model specification format
		char model_format[PARAMETER_STR_MAX];
		// output current log interval to screen
		bool show_progress;
		// time interval with which to output progress
		double progress_interval;
		// number of runs to perform
		unsigned long runs;
		// keep dividing cells
		bool keep_divisions;
		// direction of cell division
		char division_direction;
		// volume growth type
		char growth_type[PARAMETER_STR_MAX];
		// dump model to binary format
		bool dump;
		// compress hdf5 output
		bool compress;
		// hdf5 compression level (0-9; 9=best)
		unsigned int compression_level;
		// scale transport reactions with volume
		bool scale_translocation;
		// use a static lattice
		bool static_lattice;
		// don't distribute runs in MPI
		bool mpi_serial;
		// deterministic ode solver algorithm
		char ode_solver[PARAMETER_STR_MAX];
		// deterministic ode absolute error
		double ode_absolute_error;
		// deterministic ode relative error
		double ode_relative_error;
		// deterministic ode initial step size
		double ode_initial_step_size;
		// deterministic ode y scaling factor
		double ode_yscaling;
		// deterministic ode dy scaling factor
		double ode_dyscaling;
		// lattice neighbourhood (4 = von Neumann, 8 = Moore)
		unsigned int neighbourhood;

		// set a parameter value
		bool setParameter(char *name, char *value);

		// write parameter set to file
		void write(const char *filename);
};

#endif
