// simulation query definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef SIMULATIONQUERY_H
#define SIMULATIONQUERY_H

#include <hdf5.h>

namespace Simulation {

// set sbml parameter
void sbml_set_parameter(const char *model, const char *parameter, \
		double value);

// set sbml initial amount
void sbml_set_initial_amount(const char *model, const char *species, \
		const char *compartment, unsigned int amount);

// set assignment in sbml event
void sbml_set_event_assignment(const char *model, const char *species, \
		const char *event, long double value);

// set sbml compartment_volume
void sbml_set_compartment_volume(const char *model, const char *compartment, \
		double value);

// get index of species
int mcss_species_index(hid_t dfile, const char *species);

// get index of compartment
int mcss_compartment_index(hid_t dfile, const char *compartment, \
		unsigned int run);

// get average level of species
void mcss_average_level(const char *data_file, const char *species, \
		const char *compartment, double time, double *mean, double *sd);

// get average level of species for multiple timepoints
void mcss_average_level(const char *data_file, const char *species, \
		const char *compartment, double *time, unsigned int num_timepoints, \
		double *means, double *sds = NULL);

// get level of species for multiple timepoints
void mcss_level(const char *data_file, const unsigned int run, \
		const char *species, const char *compartment, double *time, \
		unsigned int num_timepoints, double *levels);

// get level of species for single timepoint
unsigned int mcss_level(const char *data_file, const unsigned int run, \
		const char *species, const char *compartment, double time);

// get number of reactions simulated
unsigned int mcss_reactions_simulated(const char *data_file);

// get simulated time for run
double mcss_simulated_time(const char *data_file, unsigned int run);

// get number of runs simulated
unsigned int mcss_num_runs(const char *data_file);

}

#endif
