// boost SimulationParameters class wrapper
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#include "mcss/mcss.h"

using namespace Simulation;

#include <boost/python.hpp>
#include <boost/python/tuple.hpp>

using namespace boost::python;

#include <string.h>

// boost declarations

void wrap_read(SimulationParameters &sp, const boost::python::list &args)
{
	unsigned int i;
	char **argv;

	// convert command line arguments
	if(!(argv = (char **) calloc(len(args), sizeof(char *)))) {
		cerr << "error converting tuple\n";
		exit(EXIT_FAILURE);
	}
	for(i = 0; i < len(args); i++) {
		string s = extract<string>(args[i]);
		if(!(argv[i] = (char *) calloc(s.size() + 1, sizeof(char)))) {
			cerr << "error converting tuple\n";
			exit(EXIT_FAILURE);
		}
		memcpy(argv[i], s.c_str(), s.size());
		argv[i][s.size()] = '\0';
	}

	// call method
	sp.read(len(args), argv);

	// tidy up
	for(i = 0; i < len(args); i++)
		free(argv[i]);
	free(argv);
}


BOOST_PYTHON_MODULE(SimulationParameters)
{
	class_<SimulationParameters>("SimulationParameters", \
			"P System simulation parameters")
		.def("read", &wrap_read, "Read parameters from a file")
		.def("set_parameter", &SimulationParameters::setParameter, \
				"Set simulation parameter")
		.def_readonly("max_time", &SimulationParameters::max_time)
		.def_readonly("duplicate_initial_amounts", \
				&SimulationParameters::duplicate_initial_amounts)
		;
}
